# This is a demo of reading CSV files and combining them to a new one
# A quite typical small utility program needed when data is received from 
# different sources in slightly different formats.
#
# CSV means Comma Separated Values and the file format can be read by spreadsheet programs
# An extra twist is the format change from decimal points to the Finnish 
# decimal comma and semicolon separator
#
# Exception checking is omitted for clarity, all files are assumed to exist
# and have expected data

# Reads a csv file of books to a list
def read_books(filename):
    books_file = open(filename, "r")
    books = []
    for book in books_file:
        book = book.rstrip()    # remove trailing newline
        if book != "":          # omit empty lines in file
            book_parts = book.split(",")
            for i in range(len(book_parts)):
                book_parts[i] = book_parts[i].rstrip('"').lstrip('"')
                    # remove quotation marks around data items,
                    # both rstrip and lstrip are applied to string
                    # Note the " inside the 's
            books.append(book_parts)
    books_file.close()
    return books

# Reads a csv file of book sales to a list
def read_book_sales(filename):
    books_file = open(filename, "r")
    booksales = []
    for book in books_file:
        book = book.rstrip()
        book_parts = book.split(",")
        for i in range(len(book_parts)):
            book_parts[i] = book_parts[i].rstrip("\"").lstrip("\"")
                # Note here that the \" is the significan quotation mark,
                # different notation from previous function
        booksales.append(book_parts)
    books_file.close()
    return booksales

def combined_book_sales(titles, sales):
    combined = []
    for i in titles:    # for each book
        for j in sales: # go through each sales record (this is not efficient for large data sets)
            if i[1] == j[0]:
                combined.append([i[1],i[2],float(i[0]),int(j[1])])
                # title, author, price, sales
    return combined

def save_sales(sales, filename):
    f_sales = open(filename, "w")
    f_sales.write('"Title";"Author";"Price";"Units";"Profit"\n')
        # The heading line, note semicolons format
    for i in sales:     # for each book
        # format price and profit to a string and change point to comma
        price = str(i[2]).replace('.',',')
        profit = str(i[2]*i[3]).replace('.',',')
        f_sales.write(f'"{i[0]}";"{i[1]}";{price};{i[3]};{profit}\n')
    f_sales.close()

books = read_books("books.csv")
booksales = read_book_sales("booksales.csv")
all_sales = combined_book_sales(books, booksales)
save_sales(all_sales, "booksalestotal.csv")
